var PO = function() {
  var string = /^\"((?:[^\"\\]|\\.)*?)\"$/;
  var comment = /^#/;
  var cmdline = /^(msgctxt|msgid|msgstr)\s+"(.*?)"$/;
  function Bundle(){this._po = []};
  Bundle.prototype = {
  add: function(obj) {
      this._po.push(obj);
    },
  byContext: function(ctxt) {
      for (var i=0, ii=this._po.length; i<ii; ++i) {
        var obj = this._po[i];
        if (obj.msgctxt && obj.msgctxt == ctxt) {
          return obj.msgstr ? obj.msgstr : obj.msgid;
        }
      }
    }
  };

  function Parser(data){
    this.iter = Iterator(data.split("\n"));
    this.line = null;
  };
  Parser.prototype = {
  doParse: function() {
      var pol = new Bundle();
      try {
        line = this.next();
        while (line != undefined) {
          // check comments
          if (this.isComment(line)) {
            line = this.next();
            continue;
          }
          var po = {};
          // check msgctx
          var [cmd, arg] = this.parseLine(line);
          if (!cmd) {
            line = this.next();
            continue;
          }
          if (cmd == 'msgctxt') {
            [str, line] = this.getString(arg);
            po.msgctxt = str;
          }
          // get msgid
          [cmd, arg] = this.parseLine(line);
          if (cmd != 'msgid') {
            this.log("msgid expected, found: " + line);
            continue;
          }
          [str, line] = this.getString(arg);
          po.msgid = str;
          // get msgstr
          [cmd, arg] = this.parseLine(line);
          if (cmd != 'msgstr') {
            this.log("msgstr expected, found: " + line);
            continue;
          }
          [str, line] = this.getString(arg);
          po.msgstr = str;
          pol.add(po);
        }
      } catch (err if err instanceof StopIteration) {
      } catch (err) {
        this.log(err);
      }
      return pol;
    },
  next: function() {
      var [i, line] = this.iter.next();
      return line;
    },
  isComment: function(line) {
      return line.match(comment)
    },
  parseLine: function(line) {
      var m = cmdline.exec(line);
      if (!m) {return [false, undefined];}
      return [m[1], m[2]]
    },
  getString: function(rest) {
      try {
        var tail = rest, line = this.next();
        var m = string.exec(line);
        while (m) {
          tail += m[1];
          line = this.next();
          m = string.exec(line);
        }
      } catch (err if err instanceof StopIteration) {line = undefined};
      return [JSON.parse('"'+tail+'"'), line];
    },
  log: Components.utils.reportError
  }
  function onLoad(data, callback) {
    var p = new Parser(data);
    var pos = p.doParse();
    if (callback) {
      callback(pos);
    }
  }
  var api = {
  parse: function(data) {
      var p = new Parser(data);
      var pos = p.doParse();
      return pos;
    },
  load: function(href, callback) {
      $.get(href, function(data, textStat){onLoad(data, callback)});
    }
  };
  return api;
}();
