const Cc = Components.classes;
const Ci = Components.interfaces;

function loadjq() {
  var subscriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"]
    .getService(Ci.mozIJSSubScriptLoader);
  var foo = subscriptLoader.loadSubScript("chrome://l10n-bug-comp/content/jquery.js");
}

var state = {
  locales: null,
  components: null
};

var check = {
  locales: function(){
    var canAdvance = false;
    var content = document.getElementById('locales').value;
    var lines = content.trim().split(/\n+/);
    state.locales = {}
    for each (var line in lines) {
      if (!line) continue;
      var match = /^([a-zA-Z\-]+)\s+(.*)/.exec(line);
      if (!match) { canAdvance = false ; break}
      canAdvance = true;
      state.locales[match[1]] = match[2];
    }
    document.getElementById('bugsWizard').canAdvance = canAdvance;
  }
};

function RDFBugConfig(instanceurl) {
  this.instance = instanceurl;
  if (!this.instance.match(/\/$/)){
    this.instance += '/';
  }
  this.vocab = "http://www.bugzilla.org/rdf#";
  this.loaded = false;
  this._RDF = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
  this._ds = this._RDF.GetDataSource(this.instance+'config.cgi?ctype=rdf&random=' + Math.floor(Math.random()*1000));
  var _loadCallback = null;
  if (!this._ds.loaded) {
    var self = this;
    this.waitForLoaded = function(cb, self, args) {
      _loadCallback = [cb, self, args];
    }
    var obs = {
    onBeginLoad: function(sink){},
    onInterrupt: function(sink){},
    onResume: function(sink){},
    onEndLoad: function(sink) {
        self.loaded = true;
        if (_loadCallback) {
          _loadCallback[0].apply(_loadCallback[1], _loadCallback[2]);
        }
      },
    onError: function(sink, stat, msg) {
        Components.utils.reportError(msg);
      }
    };
    this._ds.QueryInterface(Ci.nsIRDFXMLSink);
    this._ds.addXMLSinkObserver(obs);
  } else {
    this.loaded = true;
    this.waitForLoaded = function(cb, self, args) {
      cb.apply(self, args);
    }
  }
  this._name = this._RDF.GetResource(this.vocab + 'name');
};
RDFBugConfig.prototype.property = function(suffix) {
  return this._RDF.GetResource(this.vocab + suffix);
};
RDFBugConfig.prototype.hasProduct = function(prod) {
  if (!this.loaded) {throw "not yet loaded"};
  var obj = this._RDF.GetResource(this.instance + 'product.cgi?name=' +
                                  encodeURIComponent(prod));
  return this._ds.hasArcOut(obj, this._name);
};
RDFBugConfig.prototype.hasComponent = function(prod, comp) {
  if (!this.loaded) {throw "not yet loaded"};
  var obj = this._RDF.GetResource(this.instance + 'component.cgi?name=' +
                                  encodeURIComponent(comp) +
                                  "&product=" +
                                  encodeURIComponent(prod));
  return this._ds.hasArcOut(obj, this._name);
};


var controller = {
 size: function(_frm) {
    var contViewer = _frm.docShell.contentViewer;
    var docViewer = contViewer.QueryInterface(Components.interfaces.nsIMarkupDocumentViewer);
    docViewer.fullZoom = 0.5;
  },
 byId: function(aID) {
    return document.getElementById(aID);
  },
 alias: function(loc, thingie) {
    var sub = '';
    if (thingie) {
      sub = thingie.toLowerCase() + ".";
    }
    return loc + "@" + sub + "localization.bugs";
  },
 instance: function(page) {
    state.instance = this.byId('bz-instance').value;
    state.comps = new RDFBugConfig(state.instance);
  },
 components: function() {
    state.thingies = [];
    var page = document.getElementById('bugsWizard').currentPage;
    var _bxs =  page.querySelectorAll('checkbox');
    for (var i=0, ii=_bxs.length; i <ii; ++i) {
      var checkbx = _bxs[i];
      if (checkbx.checked) {
        state.thingies.push(checkbx.label);
      }
    }
  },
 preschemes: function(page) {
    page.parentNode.canAdvance = false;
    state.comps.waitForLoaded(function(_page) {
        _page.parentNode.canAdvance = true;
      }, page, [page]);
  },
 schemes: function(page) {
    state.products = [];
    state.components = [];
    var schemes = {}, _bxes = page.querySelectorAll('checkbox[checked=true]');
    for (var i=0, ii=_bxes.length; i < ii; ++i) {
      var cbox = _bxes[i];
      schemes[cbox.id] = this.byId(cbox.id+"-detail").value;
    }
    if ('scheme-loc' in schemes) {
      // Create a product per locale, and components per thingie
      var cat = schemes['scheme-loc'];
      for (var loc in state.locales) {
        var langName = state.locales[loc];
        var prodName = "l10n: " + langName + " / " + loc;
        var desc = langName + " localization bugs";
        if (!state.comps.hasProduct(prodName)) {
          state.products.push([cat, prodName, desc]);
        }
        for each (var thingie in state.thingies) {
            var cmpName = thingie;
            if (state.comps.hasComponent(prodName, cmpName)) {
              continue;
            }
            var cmpDesc = langName + " " + thingie + " localization";
            var qa = this.alias(loc, thingie);
            var defaultCC = [this.alias(loc)];
            state.components.push([prodName, cmpName, cmpDesc, qa, defaultCC]);
          }
      }
    }
    if ('scheme-prod' in schemes) {
      // Create a product per locale, and components per thingie
      var cat = schemes['scheme-prod'];
      for each (var thingie in state.thingies) {
          var prodName = thingie + " Localizations";
          var desc = thingie + " localization bugs";
          if (!state.comps.hasProduct(prodName)) {
            state.products.push([cat, prodName, desc]);
          }
          for (var loc in state.locales) {
            var langName = state.locales[loc];
            var cmpName = langName;
            if (state.comps.hasComponent(prodName, cmpName)) {
              continue;
            }
            var cmpDesc = langName + " " + thingie + " localization";
            var qa = this.alias(loc, thingie);
            var defaultCC = [this.alias(loc)];
            state.components.push([prodName, cmpName, cmpDesc, qa, defaultCC]);
          }
      }
    }
    if ('scheme-loc-prod' in schemes) {
      // Create a product per locale, and components per thingie
      var prodName = schemes['scheme-loc-prod'];
      for each (var thingie in state.thingies) {
          for (var loc in state.locales) {
            var langName = state.locales[loc];
            var cmpName = langName + " " + thingie + " / " + loc;
            if (state.comps.hasComponent(prodName, cmpName)) {
              continue;
            }
            var cmpDesc = langName + " " + thingie + " localization";
            var qa = this.alias(loc, thingie);
            var defaultCC = [this.alias(loc)];
            state.components.push([prodName, cmpName, cmpDesc, qa, defaultCC]);
          }
      }
    }
    var wait=6;
  },
 computeAliases: function() {
    state.aliases = {};
    for (var loc in state.locales) {
      state.aliases[loc + "@localization.bugs"] = state.locales[loc] + " watcher";
      for each (var cmp in state.thingies) {
          state.aliases[this.alias(loc, cmp)] = state.locales[loc] + " " + cmp + " watcher";
        }
    }
  },
 getAliases: function(page) {
    this.computeAliases();
    page.parentNode.canAdvance = false;
    var url = "https://landfill.bugzilla.org/l10ntest/editusers.cgi?action=list&matchvalue=login_name&matchstr=localization.bugs&matchtype=substr&groupid=12";
    var frm = document.createElement("iframe");
    frm.setAttribute("style","display:none");
    frm.setAttribute("flex","1");
    frm.setAttribute("type", "content-targetable");
    frm.addEventListener("DOMContentLoaded", controller.loadGetAliasFrame(frm), false);
    $(frm).one("DOMContentLoaded", {page:page}, controller.lga);
    document.getElementById("frms").appendChild(frm);
    frm.setAttribute('src', url);
  },
 lga: function(event) {
    var page = event.data.page;
    var foo = bar;
  },
 loadGetAliasFrame: function (frm) {
    return function (e) {
      var editNodes = this.contentDocument.querySelectorAll('a[href*="action=edit"]');
      for (var i = 0, ii = editNodes.length; i< ii; ++i) {
        var exists = editNodes[i].textContent.trim();
        if (exists in state.aliases) {
          delete state.aliases[exists];
        }
      }
      Components.utils.reportError(state.aliases.toSource());
      frm.parentNode.removeChild(frm);
      var wiz = document.getElementById('bugsWizard');
      wiz.canAdvance = true;
      wiz.advance();
    }
  },
 startCreatingAliases: function(page) {
    if (!state.aliases) {
      page.parentNode.advance();
      return;
    }
    page.parentNode.canAdvance = true; // XXX alias disabled
    var prog = this.byId('alias_progress');
    prog.max = state.aliases.length;
    prog.value = 0;
    var frm = this.byId('create_user');
    this.size(frm);
    var login, name, stack = [], progval = 0;
    function loadHandler() {
      if (stack.length) {
        var handler = stack.shift();
        handler[0].apply(frm, handler[1]);
      }
    }
    function loadEditUsers(_pv) {
      prog.value = _pv;
      this.setAttribute('src', state.instance + "/editusers.cgi?action=add");
    }
    for (login in state.aliases) {
      name = state.aliases[login];
      stack.push([loadEditUsers,[progval]]);
      stack.push([this.submitAccountData, [login, name]]);
      progval += 1;
    }
    stack.push([this.cleanLogins, [loadHandler, page]]);
    frm.addEventListener("DOMContentLoaded", loadHandler, false);
    loadHandler();
  },
 submitAccountData: function(login, name) {
    var _form;
    for each (_form in this.contentDocument.forms){
        if (_form.elements && _form.elements[0].name == "login") {
          break;
        }
      }
    _form[0].value = login;
    _form[1].value = name;
    var _pwd = '';
    for (var i=0;i<16;++i){
      _pwd += String.fromCharCode(65+Math.floor(Math.random()*26));
    }
    _form[2].value = _pwd;
    //_form.submit(); XXX alias disabled
    var _j = this;
    function _blank() {
      _j.setAttribute('src', 'about:blank');
    }
    //window.setTimeout(_blank, 1000);
  },
 cleanLogins: function(handler, page) {
    this.removeEventListener("DOMContentLoaded", handler, false);
    if (page) {
      page.parentNode.canAdvance = true;
    }
  },
 startCreatingProducts: function(page) {
    if (!state.products.length) {
      page.parentNode.advance();
      return;
    }
    page.parentNode.canAdvance = false;
    var frame = this.byId('create_prods');
    this.size(frame);
    var _page = page;
    var self = this;
    function iterProduct(index) {
      var cb = null, cbargs = null;
      if (index < state.products.length - 1) {
        cb = iterProduct;
        cbargs = [index + 1];
      } else {
        cb = function() {
          _page.parentNode.canAdvance = true;
        }
        cbargs = [];
      }
      var prod = state.products[index];
      self.addProduct(_frm, prod[0], prod[1], prod[2], iterProduct, cbargs);
    };
    iterProduct(0);
  },
 addProduct: function(frame, classification, product, description, cb, cbargs) {
    var url = "https://landfill.bugzilla.org/l10ntest/editproducts.cgi?action=add&classification=" + classification;
    var stack = [];
    function _onLoad() {
      if (stack.length) {
        var _handler = stack.shift();
        _handler[0].apply(frame, _handler[1]);
      }
    }
    function handleProduct(_p, _d) {
      var _form;
      for each (_form in document.forms){
          if (_form.elements && _form.elements[0].name=="product") {
            break;
          }
        }
      _form.elements[0].value = _p;
      _form.elements[1].value = _d;
      //_form.submit();
    }
    stack.push([handleProduct, [product, description]]);
    var self = this;
    function onDone() {
      this.removeEventListener("DOMContentLoaded", _onLoad, false);
      if (cb) {
        cb.apply(self, cbargs);
      }
    }
    stack.push([onDone, []]);
    frame.addEventListener("DOMContentLoaded", _onLoad, false);
    frame.setAttribute('src', url);
  }
};
