const Cc = Components.classes;
const Ci = Components.interfaces;

// Factor all settings-like information into a single instance
var BZL10n = {
 aliases: {'nobody@mozilla.org':
                   "Nobody; OK to take it and work on it"},
 classification: "Mozilla in Your Language",
 product: function(loc, language) {
    return "l10n: " + language + " (" + loc + ")";
  },
 components: [
  {name: "Firefox", default: true },
  {name: "Websites", default: true},
  {name: "Other", default: true},
  {name: "Thunderbird"},
  {name: "Toolkit"},
  {name: "Fennec"},
  {name: "SeaMonkey"},
  {name: "Calendar"},
              ],
 component: function (loc, comp) {
    return {component: comp,
            description: '',
            initialowner: 'nobody@mozilla.org',
            initialqacontact: this.alias(loc, comp),
            initialcc: [this.alias('all', comp), this.alias(loc)].join(','),
    };
  },
 alias: function function(loc, comp) {
    var sub = '';
    if (comp) {
      sub = comp.toLowerCase() + ".";
    }
    return loc + "@" + sub + "localization.bugs";
  }
};

// load handlers
function loadjq() {
  var subscriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"]
    .getService(Ci.mozIJSSubScriptLoader);
  var foo = subscriptLoader.loadSubScript("chrome://l10n-bug-comp/content/jquery.js");
}

var state = {
  locales: null,
  createCharts: true,
  modules: null,
  existing_aliases: [],
  aliases: {}
}, api;

var check = {
  locales: function(){
    var canAdvance = false;
    var content = document.getElementById('locales').value;
    var lines = content.trim().split(/\n+/);
    state.locales = {}
    for each (var line in lines) {
      if (!line) continue;
      var match = /^([a-zA-Z\-]+)\s+(.*)/.exec(line);
      if (!match) { canAdvance = false ; break}
      canAdvance = true;
      state.locales[match[1]] = match[2];
    }
    document.getElementById('bugsWizard').canAdvance = canAdvance;
  }
};

var logging = {
 _POLoader: function(loc, msg) {
    this.frags = loc.split('-');
    this.msg = msg;
  },
 _getMsg: function(url) {
    var msg = $('<description/>').text('Loading ' + url + ' ...');
    $('#msgs').append(msg);
    return msg;
  },
 json: function(url, data, cb) {
    var msg = this._getMsg(url);
    $.getJSON(url, data, function(json_data) {
        msg.append("Done.");
        cb(json_data);
      });
  },
 po: function(loc, cb) {
    var msg = this._getMsg("Loading POs for " + loc);
    var _pol = new this._POLoader(loc, msg);
    _pol.load(cb);
  },
 frame: function(url, cb) {
    var msg = this._getMsg(url);
    var frm = $("<iframe/>").attr({
        "style":"display:none",
          "flex":"1",
          "type": "content-targetable"});
    frm.load(function(e) {
        msg.append('Done.');
        cb(e);
        window.setTimeout(function(){frm.remove()}, 0);
      })
    $("#frms").append(frm);
    frm.attr('src', url);
    return frm;
  }
};
  
logging._POLoader.prototype = {
 load: function(cb) {
    if (cb) this.cb = cb;
    var self = this;
    var url = state.pootle.url + this.frags.join('_') + '.po';
    $.ajax({url: url,
          dataType: 'text',
          success: function(a,b,c){self.success.call(self,a,b,c)},
          error: function(a,b,c){self.error.call(self,a,b,c)}});
  },
 success: function(data) {
    this.msg.append("Done.");
    this.cb(PO.parse(data));
  },
 error: function() {
    if (this.frags.length > 1) {
      this.msg.append(" Falling back ");
      this.frags.pop();
      this.load();
    }
    else {
      this.msg.append("FAIL");
    }
  }
};

var iterProduct;

function BZFormHandler(page, list) {
  this.page = page;
  this.list = list;
  this.i = 0;
  this.page.parentNode.canAdvance = false;
  this.iterItem();
}
BZFormHandler.prototype = {
 iterItem: function() {
    if (this.i >= this.list.length) {
      this.page.parentNode.canAdvance = true;
      this.page.parentNode.advance();
      return
    }
    this.item = this.list[this.i];
    var url = this.getURL();
    this.frame = $('<iframe/>').attr({type:"content-targetable", flex:"1"});
    $(this.page).append(this.frame);
    var self = this;
    this.frame.one('load', function(){self.handleForm();self.frame.load(function(){self.onDone()});});
    this.frame.attr('src', url);
  },
 onDone: function() {
    ++this.i;
    var self = this;
    window.setTimeout(function(){self.frame.remove();delete self.frame;self.iterItem();}, 0);
  }
};

function ProductHandler(page) {
  BZFormHandler.call(this, page, state.products);
};
ProductHandler.prototype = {
 getURL: function() {
    return state.bc.url + "editproducts.cgi?action=add&classification=" + BZL10n.classification;
  },
 handleForm: function() {
    var doc = this.frame[0].contentDocument;
    var loc = this.item[0];
    var product = this.item[1];
    var description = state.po[loc].byContext("Product");
    description = description.replace("LANGUAGE", state.locales[loc]);
    var _form;
    for each (_form in doc.forms){
        if (_form.elements && _form.elements[0].name=="product") {
          break;
        }
      }
    _form.elements[0].value = product;
    _form.elements[1].value = description;
    $(_form).find('[name=createseries]').attr('checked',state.createCharts);
    _form.submit();
  }
};
ProductHandler.prototype.__proto__ = BZFormHandler.prototype;

function ComponentHandler(page) {
  BZFormHandler.call(this, page, state.components);
};
ComponentHandler.prototype = {
 getURL: function() {
    return state.bc.url + "editcomponents.cgi?action=add&product=" + this.item[1];
  },
 handleForm: function() {
    var doc = this.frame[0].contentDocument;
    var loc = this.item[0];
    var product = this.item[1];
    var component = this.item[2];
    var comp = BZL10n.component(loc, component);
    comp.description = state.po[loc].byContext(component).replace("LANGUAGE", state.locales[loc]);
    var _form;
    for each (_form in doc.forms){
        if (_form.elements && _form.elements[0].name=="component") {
          break;
        }
      }
    _form = $(_form);
    for (f in comp) {
      _form.find('[name=' + f + ']').attr('value', comp[f]);
    }
    _form[0].submit();
  }
};
ComponentHandler.prototype.__proto__ = BZFormHandler.prototype;

var controller = {
 size: function(_frm) {
    var contViewer = _frm.docShell.contentViewer;
    var docViewer = contViewer.QueryInterface(Components.interfaces.nsIMarkupDocumentViewer);
    docViewer.fullZoom = 0.5;
  },
 doCharts: function(page) {
    state.createCharts = $(page).children('checkbox').attr('checked') == 'true';
  },
 loadPOs: function() {
    state.po = {};
    function _cb(_l) {
      return function (po) {
        state.po[_l] = po;
      }
    }
    for (var loc in state.locales) {
      logging.po(loc, _cb(loc));
    }
  },
 instance: function(page) {
    state.bc = {url: $('#bz-instance')[0].value};
    state.pootle = {url: $('#pootle-instance')[0].value};
    logging.json(state.bc.url+'config.cgi', {flags:0, ctype:'json'},
                 function(data) {
                   data.url = state.bc.url;
                   state.bc = data;
                 });
    var self = this;
    state.existing_aliases = [];
    logging.frame(state.bc.url + "editusers.cgi?action=list&matchvalue=login_name&matchstr=localization.bugs&matchtype=substr", function(e) {
        self.loadGetAliasFrame(e.target);
      });
    for (ha in BZL10n.aliases) {
      logging.frame(state.bc.url + "editusers.cgi?action=list&matchvalue=login_name&matchstr=" + ha + "&matchtype=substr", function(e) {
          self.loadGetAliasFrame(e.target);
        });
    }
    api = function(method, params, cb) {
      var q = {method:method, params:params, id:1};
      var qs = JSON.stringify(q);
      $.ajax({"contentType":"application/json",
            "data": qs,
            "dataType": "json",
            "url": state.bc.url + "jsonrpc.cgi",
            "type": "post",
            success: cb
          });
    }
  },
 components: function() {
    state.modules = [];
    var page = document.getElementById('bugsWizard').currentPage;
    var _bxs =  page.querySelectorAll('checkbox');
    for (var i=0, ii=_bxs.length; i <ii; ++i) {
      var checkbx = _bxs[i];
      if (checkbx.checked) {
        state.modules.push(checkbx.label);
      }
    }
  },
 computeAliases: function() {
    state.aliases = {};
    for (var loc in state.locales) {
      state.aliases[loc + "@localization.bugs"] = state.locales[loc] + " watcher";
      for each (var cmp in state.modules) {
          state.aliases[BZL10n.alias(loc, cmp)] = state.locales[loc] + " " + cmp + " watcher";
        }
    }
    for each (var cmp in state.modules) {
        state.aliases[BZL10n.alias('all', cmp)] = "L10n " + cmp + " watcher";
      }
    $.extend(state.aliases, BZL10n.aliases);
  },
 computeProductsAndComponents: function() {
    state.products = [];
    state.components = [];
    for (var loc in state.locales) {
      var n = state.locales[loc];
      var prod = BZL10n.product(loc, n);
      if (!(prod in state.bc.product)) {
        state.products.push([loc, prod]);
      }
      var cmps = [];
      for each (var cmp in state.modules) {
          if (!(prod in state.bc.product &&
                cmp in state.bc.product[prod].component)) {
            cmps.push([loc, prod, cmp]);
          }
        }
      state.components = state.components.concat(cmps);
    }
  },
 loadGetAliasFrame: function (frm) {
    var editNodes = frm.contentDocument.querySelectorAll('a[href*="action=edit"]');
    for (var i = 0, ii = editNodes.length; i< ii; ++i) {
      var existing = editNodes[i].textContent.trim();
      state.existing_aliases.push(existing);
    }
  },
 startCreatingAliases: function(page) {
    var toBeCreated = {}, total=0;
    for (var login in state.aliases) {
      if (state.existing_aliases.indexOf(login) < 0) {
        toBeCreated[login] = state.aliases[login];
        ++total;
      }
    }
    if (!total) {
      page.parentNode.advance();
      return;
    }
    page.parentNode.canAdvance = false;
    var prog = $('#alias_progress')[0];
    prog.max = total;
    var login, name, stack = [], progval = -1;
    function loadHandler() {
      ++progval;
      prog.setAttribute("value", progval);
      if (stack.length) {
        var tpl = stack.shift();
        api('User.create', [{email:tpl[0], full_name: tpl[1]}], loadHandler);
      }
      else {
        page.parentNode.canAdvance = true;
        page.parentNode.advance();
      }
    }
    for (login in toBeCreated) {
      name = toBeCreated[login];
      stack.push([login, name]);
    }
    loadHandler();
  },
 startCreatingProducts: function(page) {
    new ProductHandler(page);
  },
 startCreatingComponents: function(page) {
    new ComponentHandler(page);
  }
};
